import json
import logging

log = logging.getLogger(__name__)


class ErrorResponse(Exception):
    def __init__(self, code, message):
        super().__init__(message)
        self.code = code


class Attribute:
    def __init__(self, name, required=False, body=True, query=False):
        self.name = name
        self.required = required
        if not body ^ query:
            raise ValueError("body or query must be exclusively true")
        self.body = body
        self.query = query


class BodyAttribute(Attribute):
    def __init__(self, name, required=False):
        super().__init__(name, required, body=True)


class QueryParameter(Attribute):
    def __init__(self, name, required=False):
        super().__init__(name, required, body=False, query=True)


class ParameterParser:
    def __init__(self, *attributes):
        self.attributes = []
        for attr in attributes:
            if not isinstance(attr, Attribute):
                raise TypeError("{} is not subtype of Attribute".format(attr.name))
        for attr in attributes:
            self.attributes.append(attr)

    def extract(self, body=None, query_params=None):
        if query_params is None:
            query_params = {}
        if body is None:
            body = {}
        args = []
        for attr in self.attributes:
            if attr.body:
                arg = body.get(attr.name)
            elif attr.query:
                arg = query_params.get(attr.name)
            else:
                arg = None
            if arg is None and attr.required:
                raise KeyError(attr.name)
            args.append(arg)
        logged = ", ".join(
            "{}={}".format(attr.name, val) for attr, val in zip(self.attributes, args)
        )
        log.info("received params: {}".format(logged))
        return args



import json
import os
import csv


class _Data:
    path = os.path.dirname(os.path.abspath(__file__))
    with open(os.path.join(path, "data", "china_cities.csv")) as csv_file:
        reader = csv.reader(csv_file)
        next(reader)
        china_cities = [row[0].lower() for row in reader]
    with open(
        os.path.join(path, "data", "china_provinces.json"), encoding="utf-8"
    ) as json_file:
        china_provinces_dict = json.load(json_file)
        china_provinces = [
            row["Province"]
            .lower()
            .replace("province", "")
            .replace("region", "")
            .replace("municipality", "")
            .replace("special administrative", "")
            .replace("autonomous", "")
            .strip()
            for row in china_provinces_dict
        ]



import re


def parse_chinese_number(number_string: str):
    TENS_OF_CHARS = {
        # https: // en.wikipedia.org / wiki / Chinese_numerals
        "一": 1,  # Normal
        "壹": 1,  # Financial / Tradination
        "十": 10,
        "拾": 10,
        "百": 100,
        "佰": 100,
        "千": 1000,
        "仟": 1000,
        "万": 10000,
        "萬": 10000,
        "亿": 100000000,
        "億": 100000000,
    }
    tens_of = 1
    for char, value in TENS_OF_CHARS.items():
        if char in number_string:
            number_string = number_string.replace(char, "")
            tens_of = tens_of * value
    number_string = re.search(r"\d+(\.\d*)?", number_string).group()
    return float(number_string) * tens_of


def get_currency(currency_str):
    currency_symbols = {"JPY": "日元", "USD": "美元", "CNY": "人民币", "EUR": "欧", "HKD": "港"}
    for k, v in currency_symbols.items():
        if v in currency_str:
            return k
    return "CNY"


def convert(from_, to):
    CACHED = {"JPY_USD": 0.0091, "EUR_USD": 1.11, "CNY_USD": 0.14, "HKD_USD": 0.13}
    return CACHED[from_ + "_" + to]



import json
from logging import getLogger

log = getLogger(__name__)


def validate_schema(Schema):
    def wrapper(func):
        schema = Schema()

        def handler(event, context):
            log.debug("received event: " + json.dumps(event))
            event = schema.load(event)
            return func(event, context=context)

        return handler

    return wrapper



import base64
import json
import os
import pickle
from uuid import uuid4
import boto3
from boto3 import client, resource
from botocore.exceptions import ClientError
from .util import json_date_converter

_BUCKET_NAME = os.getenv("BUCKET_NAME")


def translate(text, source="zh", dest="en"):
    translate_client = client(service_name="translate")
    result = translate_client.translate_text(
        Text=text, SourceLanguageCode=source, TargetLanguageCode=dest
    )
    return result.get("TranslatedText")


def get_secret(secret_id: str):
    secrets_manager = client("secretsmanager")
    get_secret_value_response = secrets_manager.get_secret_value(SecretId=secret_id)
    if "SecretString" in get_secret_value_response:
        secret = get_secret_value_response["SecretString"]
    else:
        secret = base64.b64decode(get_secret_value_response["SecretBinary"])
    return json.loads(secret)


def s3_copy(from_, to):
    s3 = resource("s3")
    copy_source = {"Bucket": _BUCKET_NAME, "Key": from_}
    bucket = s3.Bucket(_BUCKET_NAME)
    obj = bucket.Object(to)
    obj.copy(copy_source)


def s3_dump(obj, key, to_pickle=True):
    if to_pickle:
        upload_obj = pickle.dumps(obj)
    else:
        upload_obj = obj
    resource("s3").Object(_BUCKET_NAME, key).put(Body=upload_obj)


def s3_load(key, from_pickle=True):
    response = resource("s3").Object(_BUCKET_NAME, key).get()
    if from_pickle:
        return pickle.loads(response["Body"].read())
    else:
        return response["Body"].read()


def s3_exists(key):
    try:
        resource("s3").Object(_BUCKET_NAME, key).load()
        return True
    except ClientError as e:
        if e.response["Error"]["Code"] == "404":
            return False
        else:
            raise


def get_ssm_parameter(key):
    raise Exception("please activate")


def invoke_lambda(function_name, params=None, **kwargs):
    if params is None:
        params = {}
    lambda_client = client("lambda")
    res = lambda_client.invoke(
        FunctionName=function_name, Payload=json.dumps(kwargs), **params
    )["Payload"].read()
    try:
        return json.loads(res)
    except ValueError:
        return res


def add_to_queue(queue_url, body):
    msg = (
        resource("sqs")
        .Queue(queue_url)
        .send_message(
            QueueUrl=queue_url,
            MessageBody=json.dumps(body),
            MessageAttributes={
                "Recipient": {
                    "StringValue": queue_url.split("/")[-1],
                    "DataType": "String",
                }
            },
        )
    )
    return msg


def send_batch_to_queue(queue_url, bodies):
    if len(bodies) > 10:
        raise Exception("Can't send more than 10 entries at once!")
    entries = [
        {
            "Id": str(uuid4()),
            "MessageBody": json.dumps(body, default=json_date_converter),
            "MessageAttributes": {
                "Recipient": {
                    "StringValue": queue_url.split("/")[-1],
                    "DataType": "String",
                }
            },
        }
        for body in bodies
    ]
    msg = boto3.resource("sqs").Queue(queue_url).send_messages(Entries=entries)
    if msg["ResponseMetadata"]["HTTPStatusCode"] != 200:
        raise Exception(msg)
    return msg


def s3_delete(key):
    resource("s3").Object(_BUCKET_NAME, key).delete()


def stack_outputs(stack_name):
    res = resource("cloudformation").Stack(stack_name)
    return dict(i.values() for i in res.outputs)


def publish_to_sns(arn, **kwargs):
    return resource("sns").Topic(arn).publish(**kwargs)


def purge_queue(arn):
    resource("sqs").Queue(arn).purge()



import logging
import os


def init(padding=10, log_level_override=None):
    log_level = os.getenv("LOG_LEVEL", logging.INFO)
    if log_level_override is not None:
        log_level = log_level_override
    fmt = "%(asctime)s %(name)-{}s %(levelname)s: %(message)s".format(padding)
    if os.getenv("ENV") == "LOCAL":
        import coloredlogs

        coloredlogs.install(
            fmt="%(asctime)s,%(msecs)03d %(name)-{}s %(levelname)s: %(message)s".format(
                padding
            ),
            level=log_level,
        )
    else:
        root = logging.getLogger()
        if root.handlers:
            for handler in root.handlers:
                root.removeHandler(handler)
        logging.basicConfig(
            level=log_level,
            format="%(asctime)s %(name)-{}s %(levelname)s: %(message)s".format(padding),
        )



from hashlib import sha256
from base64 import b64encode


def sha256_base64(value):
    m = sha256()
    m.update(value.encode())
    bytes = m.digest()
    return str(b64encode(bytes), "utf-8")


def base36encode(integer):
    chars = "0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ"
    sign = "-" if integer < 0 else ""
    integer = abs(integer)
    result = ""
    while integer > 0:
        integer, remainder = divmod(integer, 36)
        result = chars[remainder] + result

    return sign + result



import pytest
from tiidan_utils.trade_query import _Data


@pytest.fixture
def data():
    return _Data()


def test_data(data):
    print(data.china_provinces)
    print(data.china_cities)
    assert True



import logging
from contextlib import contextmanager
from warnings import warn
import typing

try:
    import pandas as pd
except ImportError:
    warn("pandas not found")
from sqlalchemy import create_engine
from sqlalchemy.engine.url import URL
from sqlalchemy.orm import sessionmaker
import sqlalchemy as sa

log = logging.getLogger(__name__)


@contextmanager
def session_context(Session):
    session = Session()
    try:
        yield session
        session.commit()
    finally:
        session.close()


class SqlException(Exception):
    def __init__(self, message):
        super().__init__(message)
        self.message = message


class SqlHandler:
    Session = None
    engine = None

    def __init__(self, config=None, engine=None):
        if config is not None:
            self.config(config)
        if engine is not None:
            self.engine = engine
        self.Session = sessionmaker(bind=self.engine)

    @classmethod
    def from_engine(cls, engine):
        return cls(engine=engine)

    def config(self, config):
        log.debug("setting db connection to: {}".format(config["host"]))
        config = {
            "drivername": config.get("drivername", "mysql+pymysql"),
            "username": config["username"],
            "password": config["password"],
            "database": config.get("dbname", config.get("database")),
            "host": config["host"],
            "port": config.get("port", 3306),
        }
        self.engine = create_engine(URL(**config), paramstyle="pyformat")

    def raw(self, query, bindings=None):
        if bindings is None:
            bindings = {}
        with session_context(self.Session) as session:
            res = session.execute(query, bindings)
        res.close()
        return res

    def df(self, query, params=None, **kwargs):
        res = pd.read_sql(query, self.engine, params=params, **kwargs)
        return res

    def first(self, query, params=None):
        if params is None:
            params = {}
        return self.engine.scalar(query, params)

    def batch_insert_update(
        self, table, columns, values, update_columns, batch_size=100
    ):
        """
        :param table:
        :param columns:
        :param values:
        :param update_columns:
        :param batch_size:
        :return:
        """
        for row in values:
            if len(row) != len(columns):
                raise KeyError(f"All records need to have all columns'")
        for i in range(0, len(values), batch_size):
            batch = values[i : i + batch_size]
            numbered_dicts = [
                {f"{key}{i}": value for key, value in zip(columns, row)}
                for i, row in enumerate(batch)
            ]
            values_strings = [
                tuple(map(lambda x: ":" + x, d.keys())) for d in numbered_dicts
            ]
            query_values = ",".join(f"({','.join(map(str,v))})" for v in values_strings)
            bindings = {k: v for d in numbered_dicts for k, v in d.items()}
            update = ",".join(f"{col}=VALUES({col})" for col in update_columns)
            query = f"""
            INSERT INTO {table} ({','.join(columns)})
            VALUES {query_values}
            ON DUPLICATE KEY UPDATE {update}
            """
            res = self.raw(query, bindings)
            res.close()

    def list(self, query, *args, **kwargs):
        try:
            cursor = self.engine.execute(query, *args, **kwargs)
        finally:
            self.engine.dispose()
        res_list = [s[0] for s in cursor]
        cursor.close()
        return res_list

    def dispose(self):
        self.engine.dispose()

    def insert_on_duplicate_update(
        self,
        model,
        values: typing.List[typing.Dict[str, typing.Any]],
        columns_to_update,
    ):
        columns_to_update = [
            model.__getattribute__(model, v).name for v in columns_to_update
        ]
        insert_statement = sa.insert(model).values(values)
        return engine.execute(
            insert_statement.on_duplicate_key_update(
                **{
                    v: insert_statement.inserted.__getattr__(v)
                    for v in columns_to_update
                }
            )
        )



try:
    from flask import request
except ImportError:
    raise ImportError("Need flask for api utils")

from logging import getLogger
from functools import wraps
from marshmallow import ValidationError
from werkzeug.exceptions import BadRequest

log = getLogger(__name__)


def load_schema(Schema, from_="body"):
    def decorator(f):
        @wraps(f)
        def inner():
            try:
                if from_ == "query":
                    log.debug(request.args)
                    args = Schema().load(request.args)
                else:
                    log.debug(request.json)
                    args = Schema().load(request.json)
                return f(*args)

            except ValidationError as err:
                log.error(err.messages)
                raise BadRequest(err.messages)

        return inner

    return decorator



import re
from datetime import datetime, date
from functools import wraps


def now_string():
    return datetime.now().isoformat().split(".")[0].replace(":", "").replace("-", "")


def remove_symbols_from_supplier(string):
    string = re.sub(r"[^a-zA-Z0-9&]", " ", string)
    string = re.sub(r"\s+", " ", string)
    string = string.strip()
    string = string.lower()
    return string


def camel_case_to_snake_case(str):
    return re.sub(r"(?<=[a-z])[A-Z]|[A-Z](?=[^A-Z])", r"_\g<0>", str).lower().strip("_")


def log_function(logger):
    def wrapper(func):
        @wraps(func)
        def inner(*args, **kwargs):
            logger.debug("args: " + str(args))
            logger.debug("kwargs: " + str(kwargs))
            res = func(*args, **kwargs)
            logger.debug("result: " + str(res))
            return res

        return inner

    return wrapper


def json_date_converter(o):
    if isinstance(o, (date, datetime)):
        return o.isoformat()


def null_on_error(return_value):
    def wrapper(func):
        @wraps(func)
        def inner(*args, **kwargs):
            try:
                return func(*args, **kwargs)
            except:
                return return_value

        return inner

    if callable(return_value):
        return null_on_error(None)(return_value)
    return wrapper



import json

import pandas as pd
import pytest
from sqlalchemy import create_engine
from sqlalchemy.engine.url import URL
from tiidan_utils.sql_handler import SqlHandler


@pytest.fixture
def sql():
    with open("secrets/db_local.json") as f:
        config = json.load(f)
    engine = create_engine(
        URL(
            **{
                "drivername": config.get("drivername", "mysql+pymysql"),
                "username": config["username"],
                "password": config["password"],
                "database": config.get("dbname", config.get("database")),
                "host": config["host"],
                "port": config.get("port", 3306),
            }
        )
    )
    return SqlHandler(engine=engine)


@pytest.fixture
def db_context(sql):
    sql.raw(
        """
    create table test_insert_table
    (
        id int not null,
        name VARCHAR(256) null,
        animal VARCHAR(256) null,
        money int null,
        constraint test_insert_update_pk
            primary key (id)
    );
    """
    )
    sql.raw(
        """
    INSERT INTO test_insert_table (id, name, animal, money)
    VALUES (0, 'tom', 'cat', 100), (1, 'maya', 'dog', 200), (2, 'larry', 'snake', 300)
    """
    )
    yield
    sql.raw(
        """
        DROP TABLE test_insert_table
        """
    )


def test_insert_update(db_context, sql):
    records = [(1, "tom", "elephant")]
    sql.batch_insert_update(
        "test_insert_table", ("id", "name", "animal"), records, ["animal"]
    )
    animal = sql.first("SELECT animal from test_insert_table where id=1")
    assert animal == "elephant"

    records = [(0, "tom", "mouse", 20000)]
    sql.batch_insert_update(
        "test_insert_table", ("id", "name", "animal", "money"), records, ["money"]
    )
    animal = sql.first("SELECT animal from test_insert_table where id=0")
    money = sql.first("SELECT money from test_insert_table where id=0")
    assert animal == "cat"
    assert money == 20000

    records = [(0, "tom", "mouse", 20000), (3, "jerry", "giraffe", -100)]
    sql.batch_insert_update(
        "test_insert_table",
        ("id", "name", "animal", "money"),
        records,
        ["money", "animal"],
    )
    jerry = sql.first("SELECT name from test_insert_table where id=3")
    assert jerry == "jerry"


def test_df(db_context, sql: SqlHandler):
    res = sql.df("select animal from test_insert_table")
    assert isinstance(res, pd.DataFrame)
    assert len(res) == 3
    assert "cat" in res.animal.tolist()
    res = sql.df("select animal from test_insert_table where animal != %s", params=["cat"])
    assert "cat" not in res.animal.tolist()


def test_list(db_context, sql: SqlHandler):
    res = sql.list("select animal from test_insert_table")
    assert isinstance(res, list)
    assert len(res) == 3
    assert "cat" in res
    res = sql.list("select animal from test_insert_table where animal != %s", "cat")
    assert "cat" not in res



import json

import pytest
from elasticsearch_dsl import Search

from tiidan_utils.trade_query.trade_search import (
    get_supplier_query,
    parse_structure,
    DomesticChineseCompany,
    UncategorizedChineseCompany,
)


@pytest.mark.parametrize(
    "supplier,expected_class",
    (
        ("ANHUI LTECH INFORMATION TECHNOLOGY CO., LTD.", DomesticChineseCompany),
        ("HARBIN TENGZHAN WOOD PRODUCTS CO.,LTD", DomesticChineseCompany),
        ("Anhui technology import export", DomesticChineseCompany),
        ("China Guoxin Holdings Co., Ltd.", UncategorizedChineseCompany),
        ("China Guoxin Holdings Co., Ltd.", UncategorizedChineseCompany),
        ("DaLian YiLian Trade Co.Ltd", DomesticChineseCompany),
        ("Nanjing Co Energy Optical Crystal Co Ltd ", DomesticChineseCompany),
        ("Zhejiang Ziguang Materials Trading Co ", DomesticChineseCompany),
        ("SHENZHEN COREPIN MOULD CO LTD", DomesticChineseCompany),
    ),
)
def test_parse_structure(supplier, expected_class):
    print("supplier: ", supplier)
    res = parse_structure(supplier)
    assert res is not None
    assert isinstance(res, expected_class)
    if expected_class == DomesticChineseCompany:
        print(res.location)
        print(res.rest)
        print(res.type)


@pytest.mark.parametrize(
    "supplier, field",
    (
        ("HARBIN TENGZHAN WOOD PRODUCTS CO.,LTD", "Exporter"),
        ("Anhui technology import export", "Exporter"),
        ("China Guoxin Holdings Co., Ltd.", "Exporter"),
        ("China Guoxin Holdings Co., Ltd.", "CUSTOM_FIELD"),
        ("DaLian YiLian Trade Co.Ltd", "Exporter"),
        ("Nanjing Co Energy Optical Crystal Co Ltd ", "Exporter"),
    ),
)
def test_trade_query(supplier, field):
    print("supplier: ", supplier)
    res = get_supplier_query(supplier, field)
    assert res is not None
    print(f"result: {res}")
    print("-----------------------------")


def parse_supplier(supplier):
    print("supplier: ", supplier)
    q = parse_structure(supplier)
    print(q)
    print("-----------------------------")


@pytest.mark.skip
def test_parse_co_ltd():
    q = parse_structure("HARBIN TENGZHAN WOOD PRODUCTS CO.,LTD")
    print(q.to_dict())


@pytest.mark.skip
def test_parse_conbest():
    q = parse_structure("Gansu Conbest Biotech Co., Ltd.")
    print(q.to_dict())


@pytest.mark.skip
def test_parse_lideer():
    q = parse_structure(" Jiangsu Lideer Pneumatic Tools Company Limited")
    print(q.to_dict())


@pytest.fixture
def elasticsearch_conn():
    from elasticsearch import Elasticsearch
    from elasticsearch_dsl import connections

    with open("secrets/elasticsearch_prod.json") as f:
        elasticsearch_config = json.load(f)
    client = Elasticsearch(
        **elasticsearch_config, verify_certs=False, use_ssl=True, timeout=120
    )
    connections.add_connection("default", client)


@pytest.mark.parametrize(
    "supplier,shipments",
    (
        ("HARBIN TENGZHAN WOOD PRODUCTS CO.,LTD", 1),
        ("Anhui technology import export", 1),
        ("China Guoxin Holdings Co., Ltd.", 0),
        ("DaLian YiLian Trade Co.Ltd", 0),
        ("Nanjing Co Energy Optical Crystal Co Ltd", 1),
    ),
)
def test_real_search(supplier, shipments, elasticsearch_conn):
    res = (
        Search(index="consolidated_trade")
        .from_dict(get_supplier_query(supplier))
        .execute()
    )
    assert res.hits.total >= shipments



import logging
import re
import traceback
from typing import List
from elasticsearch_dsl import Search
from elasticsearch_dsl.query import (
    QueryString,
    Bool,
    Match,
    SpanNear,
    SpanMulti,
    SpanFirst,
    SpanTerm,
    SpanOr,
)

from tiidan_utils.util import log_function
from .data import _Data

data = _Data()
log = logging.getLogger(__name__)


@log_function(log)
def get_supplier_query(name, field="Exporter"):
    return Search().query(parse_structure(name).get_query(field)).to_dict()


class Replacer:
    def __init__(self, pattern, query):
        self.pattern = pattern
        self.query = query


class _Fuzzy(Match):
    def __init__(self, term, field):
        super().__init__(**{field: dict(fuzziness="AUTO", query=term, prefix_length=1)})


class _QueryString(QueryString):
    def __init__(self, query, field):
        super().__init__(default_field=field, query=query)


class _SpanMultiFuzzy(SpanMulti):
    def __init__(self, query, field):
        super().__init__(
            match={
                "fuzzy": {
                    field: {"value": query, "fuzziness": "AUTO", "prefix_length": 1}
                }
            }
        )


def regexes(field):
    return [
        Replacer(r"\bproducts?\b", _Fuzzy("product", field)),
        Replacer(r"\bim?p?o?r?t?\s*ex?p?o?r?t?\b", _QueryString("i* e*", field)),
        Replacer(r"\s+xing\b", _QueryString("*xing", field)),
        Replacer(r"\s+lumin\b", _QueryString("*lumin", field)),
        Replacer(
            r"\bindustry?i.?a?l?\b", _QueryString("industrial~ OR industry~", field)
        ),
        Replacer(r"\bcompany\b", _QueryString("-company", field)),
        Replacer(r"\beconomy?i?c?\b", _QueryString("economy~ OR economic~", field)),
        Replacer(r"\binstru?m?e?n?t?s?a?l?\b", _QueryString("istru*", field)),
        Replacer(r"\btrade?(ing)?\b", _QueryString("trade~ OR trading~", field)),
    ]


def clean_name(name):
    regexes = [
        {"pattern": r"[\[\]()\\ /:;\'.,`-]"},
        {"pattern": r"\n"},
        {"pattern": r"co\s*ltd?"},
        {"pattern": r"(limited)|(\blt?d?)|(llc)\b"},
        {"pattern": r"\bcor?p?o?r?a?t?i?o?n?\b"},
        {"pattern": r"\binco?r?p?o?r?a?t?e?d?\b"},
        {"pattern": r"\bsha\b"},
        {"pattern": r"\bbldg\b"},
        {"pattern": r"\band\b"},
        {"pattern": r"&"},
        {"pattern": r"\bgmbh\b"},
        {"pattern": r"\bno\b"},
        {"pattern": r"\s+"},
    ]
    trade_search = name.lower()
    for r in regexes:
        trade_search = re.sub(r["pattern"], r.get("replace", " "), trade_search)
    return trade_search.strip()


def get_type_query(tokens, field):
    regexes = {
        r"li?m?i?te?d": lambda x: SpanOr(
            clauses=[_SpanMultiFuzzy("limited", field), _SpanMultiFuzzy("ltd", field)]
        ),
        "cor?p?o??r?a?t?i?o?n?": lambda x: SpanOr(
            clauses=[
                _SpanMultiFuzzy("corp", field),
                _SpanMultiFuzzy("co", field),
                _SpanMultiFuzzy("corporation", field),
            ]
        ),
        "default": lambda x: _SpanMultiFuzzy(x, field),
    }
    clauses = []
    for token in tokens:
        for reg, term in regexes.items():
            if re.search(reg, token):
                clauses.append(term(token))
                break
        else:
            clauses.append(regexes["default"](token))
    return SpanNear(clauses=clauses, in_order=True, slop=0)


class ChineseCompany:
    pass


class DomesticChineseCompany:
    def __init__(self, tokens):
        self.location = tokens[0]
        log.debug(f"location: {self.location}")
        tokens = tokens[1:]
        type_index = self.get_type_index(tokens)
        self.type = []
        if type_index:
            self.type = tokens[type_index : type_index + 2]
            log.debug(f"type: {', '.join(self.type)}")
            self.rest = tokens[:type_index]
        else:
            self.rest = tokens[:]
        log.debug(f"rest: {', '.join(self.rest)}")

    @staticmethod
    def get_type_index(tokens: List[str]) -> int:
        for i, t in enumerate(tokens):
            if t == "co" and (
                i + 1 == len(tokens)
                or re.match(
                    r"\b((ltd?)|(inco?r?p?o?r?a?t?ed?)"
                    r"|(llc)|(limited)|(liability)|(company)"
                    r"|(LIMITED))\b",
                    tokens[i + 1],
                )
            ):
                return i
            if re.match(
                r"\b((corpo?r?a?t?i?o?n?)|(ltd?)|(inco?r?p?o?r?a?t?ed?)"
                r"|(llc)|(limited)|(liability)|(company)"
                r"|(LIMITED))\b",
                t,
            ):
                return i
        return None

    def get_query(self, field="Exporter"):
        if len(self.rest) == 0:
            return None
        else:
            clauses = [SpanTerm(**{field: self.location})] + [
                _SpanMultiFuzzy(x, field) for x in self.rest
            ]
            if len(self.type):
                clauses.append(get_type_query(self.type, field))

            return Bool(
                must=[
                    SpanFirst(match=SpanTerm(**{field: self.location}), end=1),
                    SpanNear(clauses=clauses, in_order=True, slop=0),
                ]
            )


class ForeignOwnedChineseCompany:
    pass


class NationallyOwnedChineseCompany:
    pass


class UncategorizedChineseCompany:
    def __init__(self, tokens):
        self.tokens = tokens

    def get_query(self, field="Exporter"):
        return get_type_query(self.tokens, field)


def tokenize(name: str) -> List[str]:
    name = re.sub(r"[\[\]()\\ /:;\'.,`\-\"\}&]", " ", name)
    name = re.sub(r"\s+", " ", name)
    name = name.strip()
    return [x.lower() for x in name.split(" ")]


def parse_structure(name):
    tokens = tokenize(name)
    for i, t in enumerate(tokens):
        if i == 0 and (t in data.china_provinces or t in data.china_cities):
            try:
                return DomesticChineseCompany(tokens)
            except Exception as e:
                log.warning(traceback.format_exc())
                log.warning(f"couldn't parse name: {name}")
    return UncategorizedChineseCompany(tokens)



import logging
from tiidan_utils.util import log_function


def test_function_logger():
    @log_function(logging)
    def foo(*args, **kwargs):
        return "hello"

    args = [1, "currency_str"]
    kwargs = {"key": "value"}
    foo(*args, **kwargs)



import json
import logging
from copy import copy
from functools import wraps
from typing import Callable
from tiidan_utils.aws_utils import add_to_queue

log = logging.getLogger(__name__)


def send_to_next(results, next_tasks):
    if not isinstance(results, list):
        results = [results]
    for task in next_tasks:
        for result in results:
            log.info(f"sending to {task}:" + str(result))
            add_to_queue(task, result)


def sqs_handler(batch=False):
    func = None
    if callable(batch):
        func = copy(batch)
        batch = False

    def wrapper(handler):
        @wraps(handler)
        def _handler(event, context):
            log.debug("event: " + str(event))
            if not event.get("Records"):
                log.info("invoking as lambda with event: " + str(event))
                return handler(event, context)

            if batch:
                records = [json.loads(record["body"]) for record in event["Records"]]
                log.info("invoking as batch: " + str(records))
                handler({"records": records}, context)
            else:
                log.info("invoking sqs handler one by one")
                for record in event["Records"]:
                    body = json.loads(record["body"])
                    log.info("processing record: " + str(body))
                    handler(body, context)

        return _handler

    if callable(func):
        return wrapper(func)
    return wrapper


def s3_handler(func: Callable[[str, str], object]):
    """
    :param func: receives (bucket, key) as arguments
    :return:
    """

    def handler(event, context):
        for record in event["Records"]:
            return func(record["s3"]["bucket"]["name"], record["s3"]["object"]["key"])

    return handler



import logging
from click.testing import CliRunner
from tiidan_utils.cli import main
from tiidan_utils.util import log_function


def test_main():
    runner = CliRunner()
    result = runner.invoke(main, [])

    assert result.output == "()\n"
    assert result.exit_code == 0



import json
import pytest

from tiidan_utils.lamda_decorators.handlers import sqs_handler

lambda_event = {"key": "value"}
sqs_event = {"Records": [{"body": json.dumps({"key": "value"})}]}


def handler(event, context=None):
    return event["key"]


@pytest.mark.parametrize("event", [lambda_event, sqs_event])
def test_sqs_handler(event):
    return sqs_handler(handler)(event, None)



import logging
from .data import _Data
from .trade_search import get_supplier_query

log = logging.getLogger(__name__)



def lambda_handler(event, context=None):
    name = (
        event.get("name")
        or event.get("supplier")
        or event.get("englishName")
        or event.get("english")
        or event.get("english_name")
    )
    if name is None:
        raise ValueError("Please specify name")
    log.info(f"received: {name}")
    res = get_supplier_query(name)
    log.info(f"returning: {res}")
    return res
